module Passety
  class Wrapper
    def initialize(token, options = {})
      @token = token
      @raise_errors = options.fetch(:raise_errors, true)
    end

    def collections
      @collections ||= Collection.new(self)
    end

    def things
      @things ||= Thing.new(self)
    end

    def uploads
      @uploads ||= Upload.new(self)
    end

    def search(name, options = {})
      perform_request { |c| c.get('/explore', options.merge(q: name)) }
    end

    def info
      perform_request { |c| c.get('/') }
    end

    def perform_request
      response = Response.new(yield(connection))

      if @raise_errors
        raise NotAuthorizedError.new if response.status == 401
        raise ReadOnlyError.new      if response.status == 403
        raise NotFoundError.new      if response.status == 404
      end

      response
    end

    private
    def connection
      @connection ||= Faraday.new(url: API_URL, headers: { 'Accept' => 'application/json; version=1', 'Authorization' => "Bearer #{@token}" }) do |f|
        f.request :multipart
        f.adapter Faraday.default_adapter
      end
    end
  end
end
